#include "rerefons.h"
#include <QDebug>
#include <QTimer>

RereFons::RereFons(QObject *parent) : QObject(parent)
{
    m_nCmpt = m_nSlider = 0;

    m_client = new QMqttClient(this);
    m_client->setHostname(MQTT_BROKER);
    m_client->setPort(MQTT_PORT);

    connect(m_client, &QMqttClient::stateChanged, this, &RereFons::updateLogStateChange);
    connect(m_client, &QMqttClient::disconnected, this, &RereFons::brokerDisconnected);
    connect(m_client, &QMqttClient::messageReceived, this, [this](const QByteArray &message, const QMqttTopicName &topic) {
        const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(" Received Topic: ")
                    + topic.name()
                    + QLatin1String(" Message: ")
                    + message
                    + QLatin1Char('\n');
        vGestionaResposta(topic.name(),message);
        //ui->plainTextEdit->insertPlainText(content);
    });

    if (m_client->state() == QMqttClient::Disconnected) {
        bMqttConnectat = false;
        m_client->connectToHost();
    }

    QTimer *timer = new QTimer(this);
    connect(timer,SIGNAL(timeout()),
            this,SLOT(vActualitza()));
    timer->start(50);
}

QString RereFons::szCadena(){
    return m_szCadena;
}

void RereFons::setCadena(const QString &szValue){
    m_szCadena = szValue;

    qDebug() << "Des de C++: " << szValue;
    emit cadenaCanviada();
}

int RereFons::nCmpt(){
    return m_nCmpt;
}

void RereFons::setCmpt(const int &nC){
    m_nCmpt = nC;

    emit cmptChanged();
}


int RereFons::nSlider(){
    return m_nSlider;
}

void RereFons::setSlider(const int &nC){
    m_nSlider = nC;

    qDebug() << "Des de C++ es llegeix el valor de l'slider QML: " << m_nSlider;

    emit sliderChanged();
}

void RereFons::vActualitza(){
    QString qsMissatge;
    m_nCmpt++;
    if(m_nCmpt > 100)
        m_nCmpt = 0;
    emit cmptChanged();
    //qsMissatge = qsTr8("%1").arg(m_nCmpt);
    if(bMqttConnectat)
        //publica(TEMA_T3,QString::number(m_nCmpt));
        publica(TEMA_T3,QString::number(m_nSlider));
}

void RereFons::vGestionaResposta(QString tema,QString missatge){
    if(tema == TEMA_T1){
        //ui->progressBarT1->setValue(missatge.toInt());
    }
    if(tema == TEMA_CABAL){
        setCadena(missatge);
    }
}

void RereFons::publica(QString tema,QString missatge)
{
    if (m_client->publish(tema, missatge.toUtf8()) == -1){
        qDebug() << "MQTT error: Could not publish message";
    }
}

void RereFons::updateLogStateChange()
{
    int nEstat = -1;
    const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(": State Change")
                    + QString::number(nEstat = m_client->state())
                    + QLatin1Char('\n');
    qDebug() << content;
    if(nEstat == 2){
        subscription(TEMA_T1);
        subscription(TEMA_CABAL);
        bMqttConnectat = true;
    }
}

void RereFons::brokerDisconnected()
{
    qDebug() << "Broker disconnected!";
}

void RereFons::subscription(QString topic)
{
    auto subscription = m_client->subscribe(topic);
    if (!subscription) {
        qDebug() << "MQTT Error: Could not subscribe. Is there a valid connection?";
        return;
    }else
        qDebug() << "Subscrit a " << topic;
}
